/***** BEGIN LICENSE BLOCK *****
   - Version: MPL 1.1/GPL 2.0/LGPL 2.1
   -
   - The contents of this file are subject to the Mozilla Public License Version
   - 1.1 (the "License"); you may not use this file except in compliance with
   - the License. You may obtain a copy of the License at
   - http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS IS" basis,
   - WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
   - for the specific language governing rights and limitations under the
   - License.
   -
   - The Original Code is the View Background Plus.
   -
   - The Initial Developer of the Original Code is Ingo Rau.
   - Portions created by the Initial Developer are Copyright (C) 2004
   - the Initial Developer. All Rights Reserved.
   -
   - Contributor(s): Ingo Rau <ingorau @ imail.de>
   -
   - Thanks to Philip Mateescu (Copy URL+), whose extension I borrowed
   - as framework for my first own extension. There's nothing left of
   - his code though, save some generic stuff and the overall structure
   - (which also is rather generic for Mozilla extensions).
   -
   - Alternatively, the contents of this file may be used under the terms of
   - either the GNU General Public License Version 2 or later (the "GPL"), or
   - the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
   - in which case the provisions of the GPL or the LGPL are applicable instead
   - of those above. If you wish to allow use of your version of this file only
   - under the terms of either the GPL or the LGPL, and not to allow others to
   - use your version of this file under the terms of the MPL, indicate your
   - decision by deleting the provisions above and replace them with the notice
   - and other provisions required by the LGPL or the GPL. If you do not delete
   - the provisions above, a recipient may use your version of this file under
   - the terms of any one of the MPL, the GPL or the LGPL.
   -
   - ***** END LICENSE BLOCK *****/

var quicksavedir=null;


function error(id)
{
 try
 { alert("ViewBgPlus: "+string_bundle.getString("error_"+id)); }
 catch (ex)
 { alert("ViewBgPlus: "+string_bundle.getString("error_generic")); }
}


/**
 * Returns the background image set for an HTML element.
 * Can be either CSS style (background-image=...) or standard HTML attribute
 * (background=...). CSS overrides standard HTML.
 */
function getBackgroundImageDefinition(obj)
{
 // The parent node of <HTML> is "undefined", which _is_ a valid object,
 // but obj.style is not defined.
 if (obj.style)
 {
  if (obj.style.backgroundImage)
  {
   var file = String(obj.style.backgroundImage);
   return file.substring(4,file.length-1);
  }
  else
  {
   // Wonder why this doesn't work for background, but for others (see below)
   //return obj.getAttribute("background");
   for (i=0;i<obj.attributes.length;i++)
   { 
    if (obj.attributes[i].nodeName=="background")
    { return obj.attributes[i].nodeValue; }
   }
  }
 }
 return "";
}


/**
 * Returns the background image set for the current HTML element
 * or any parent element
 */
function getBackgroundImage(obj)
{
 while ((getBackgroundImageDefinition(obj)=="") && (obj.style))
 { obj = obj.parentNode }
 return getBackgroundImageDefinition(obj)
}


/**
 * Returns the actual tag the background image is defined in
 */
function getBackgroundImageTag(obj)
{
 while ((getBackgroundImageDefinition(obj)=="") && (obj.style))
 { obj = obj.parentNode }
 return obj;
}


/**
 * Expands relative URL, using a reference URL (which has to be fully qualified)
 * Couldn't find a built-in way to do this (any suggestions?)
 *
 * Attention!
 * This is very straight-forward, simple way, may not work for "non-standard"
 * URLs! (like "pix/../../index/pictures" or such stupid stuff)
 */
function expandRelativeURL(url,href)
{
 if (!url.match(/.*:\/.*/))
 {
  href = href.substr(0,href.lastIndexOf("/"));
  while (url.substr(0,3)=="../")
  {
   href = href.substr(0,href.lastIndexOf("/"));
   url = url.substr(3,url.length);
  }
  if (url.charAt(0)=="/")
  { url = url.substr(1,url.length); }
  return href+"/"+url;
 }
 else
 { return url; }
} 
 
 
/**
 * Quick saves any URL. Let's the user choose a quicksave directory if this
 * wasn't done before, in that case just save to that directory.
 */
function doQuicksave(url,obj)
{
 if (quicksavedir==null)
 {
  var filepicker = Components.classes["@mozilla.org/filepicker;1"].createInstance().
                   QueryInterface(Components.interfaces.nsIFilePicker);
  filepicker.init(getBrowser.contentWindow,
                  string_bundle.getString("dirpicker_title"),
                  filepicker.modeGetFolder);
  filepicker.appendFilters(Components.interfaces.nsIFilePicker.filterAll);
  if (filepicker.show() == Components.interfaces.nsIFilePicker.returnOK)
  { quicksavedir=filepicker.file; }
 }    

 // Do the download
 var data = {url: url, fileName: null, filePickerTitle: null, document: null, 
             byPassCache: false, referrer: getBrowser().contentDocument.URL,
	     directory: quicksavedir.path, obj: obj};
 var sniffer = new nsHeaderSniffer(url,viewbgSniffer,data);
}


/**
 * Internally used for the actual download/save of the image
 */
function viewbgSniffer(aSniffer,aData)
{
 var downloadOK=true;

 var defaultName = getDefaultFileName(aData.fileName,aSniffer.suggestedFileName,aSniffer.uri,null);
 var lf = Components.classes["@mozilla.org/file/local;1"].
          createInstance(Components.interfaces.nsILocalFile);
 lf.initWithPath(aData.directory);
 lf.append(defaultName);
 if (lf.exists())
 {
  var i=0;
  var exp=10;
  var name=defaultName.substr(0,defaultName.lastIndexOf("."));
  var ext=defaultName.substr(defaultName.lastIndexOf("."),defaultName.length);
  var index="000";
  do
  {
   lf.initWithPath(aData.directory);
   lf.append(name+"_"+index+ext);
   if (i++ > 999) // Avoid infinite recursion (for whatever reasons)
   {              // Should do this with preference
    error("novalidfilename");
    downloadOK=false;
    break;
   }
   // The easy way...
   if (i<10)
   { index = "00"+i; }
   else if (i<100)
   { index = "0"+i; }
   else
    index = String(i);    
  } while (lf.exists())
 }

 if (downloadOK)
 {
  var src=aSniffer.uri;
  var persistArgs={source: src, contentType: aSniffer.contentEncodingType,
                   target: makeFileURL(lf), postData: null, bypassCache: false};
  var persist=makeWebBrowserPersist();
  const nsIWBP=Components.interfaces.nsIWebBrowserPersist;
  const flags=nsIWBP.PERSIST_FLAGS_NO_CONVERSION |
              nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES;
  persist.persistFlags=flags | ((aData.bypassCache)?(nsIWBP.PERSIST_FLAGS_BYPASS_CACHE)
	                                           :(nsIWBP.PERSIST_FLAGS_FROM_CACHE));
  var download=Components.classes["@mozilla.org/download;1"].
               createInstance(Components.interfaces.nsIDownload);
  try
  {
   download.init(src,persistArgs.target,null,null,null,persist);
   persist.saveURI(src,null,makeURL(aData.referrer),persistArgs.postData,
                   "ViewBgPlus: 0.9\r\n",persistArgs.target);
   downloadOK=true;
  }
  catch (ex)
  { downloadOK=false; }
 } 

 // Mark saved element
 var prefs = Components.classes["@mozilla.org/preferences-service;1"].
	     getService(Components.interfaces.nsIPrefBranch);
 var mark = true;
 try
 { mark = prefs.getBoolPref("viewbgplus.mark_quicksave"); }
 catch (ex) {}
 if (mark)
 {
  var style = "1px dotted";
  try
  { style = prefs.getCharPref("viewbgplus.mark_quicksave.style"); }
  catch (ex) {}
  if (downloadOK)
  { aData.obj.style.border=style+" green"; }
  else
  { aData.obj.style.border=style+" red"; }
 } 
}


/**
 * Menu item handler for saveing the  background image of the 
 * current HTML element or any parent
 */
function doSaveBackgroundImage()
{
 var b = getBrowser();
 if (gContextMenu && b)
 {
  var obj = gContextMenu.target;
  if (obj)
  {
   var url = expandRelativeURL(getBackgroundImage(obj),b.contentWindow.location.href);
   saveURL(url,null,null,true);
  }
 }
}


/**
 * Menu item handler for quick saving the background image.
 * Essentially calls doQuicksave()
 */
function doQuickSaveBackgroundImage()
{
 var b = getBrowser();
 if (gContextMenu && b)
 {
  var obj = gContextMenu.target;
  if (obj)
  {
   var url = expandRelativeURL(getBackgroundImage(obj),b.contentWindow.location.href);
   doQuicksave(url,getBackgroundImageTag(obj));
  }
 }
}


/**
 * Menu item handler for quick saving the image from <IMG>.
 * Essentially calls doQuicksave()
 */
function doQuickSaveImage()
{
 var b = getBrowser();
 if (gContextMenu && b)
 {
  var obj = gContextMenu.target;
  if (obj && (obj.tagName.match(/IMG/i))) // should be an IMG tag...
  {
   var url = expandRelativeURL(obj.getAttribute("src"),b.contentWindow.location.href);
   doQuicksave(url,obj);
  }
  else // ...because menu entry will be disabled/hidden if not
  { error("generic"); }
 }
}

       
/**
 * Menu item handler for opening background image of the current HTML
 * element or any parent in new tab, new window or same window (depending
 * on preference)
 */
function doViewBackgroundImage()
{
 var prefs = Components.classes["@mozilla.org/preferences-service;1"].
	     getService(Components.interfaces.nsIPrefBranch);
 var prefVal;
 var b = getBrowser();

 if (gContextMenu && b)
 {
  var obj = gContextMenu.target;
  if (obj)
  {
   var url = expandRelativeURL(getBackgroundImage(obj),b.contentWindow.location.href);
   try
   { prefVal = prefs.getIntPref("viewbgplus.target"); }
   catch (ex)
   { prefVal = 0; }
   switch (prefVal)
   {
    case 1: b.contentWindow.open(url,url);
            break;
    case 2: b.contentWindow.location.href = url;
            break;
    default: var loadInBackground = true;
             try
             { loadInBackground = prefs.getBoolPref("browser.tabs.loadInBackground"); }
             catch (ex) {}
             var tab = b.addTab(url);
             if (!loadInBackground)
             { b.selectedTab = tab; }
   }
  }
 }
}


/**
 * Event listener for showing of context menu:
 * Adjust menu items according to preferences
 */
function viewbgplusContext()
{
 var prefs = Components.classes["@mozilla.org/preferences-service;1"]. 
	     getService(Components.interfaces.nsIPrefBranch);
 var removeBuiltIn = true;
 var hide_disable = true;
 var showbg = false;
 var showimg = false;
 var show_view = true;
 var show_save = true;
 var show_qsave = true;

 if (gContextMenu)
 {
  var obj = gContextMenu.target;
  if (obj)
  {
   if (obj.tagName.match(/IMG/i))
   { showimg = true; }
   while ((getBackgroundImage(obj)=="") && (obj.style))
   { obj = obj.parentNode }
   var url = getBackgroundImage(obj)
   if (url!="")
   { showbg = true; }
  }

  // Set status of own menu items
  try
  { hide_disable = prefs.getBoolPref("viewbgplus.hide_menuitems"); }
  catch (ex) {}
  var attr = (hide_disable)?("hidden"):("disabled");
  try
  { show_view = prefs.getBoolPref("viewbgplus.show_view"); }
  catch (ex) {}
  try
  { show_save = prefs.getBoolPref("viewbgplus.show_save"); }
  catch (ex) {}
  try
  { show_qsave = prefs.getBoolPref("viewbgplus.show_quicksave"); }
  catch (ex) {}
  
  var menuitem = document.getElementById("viewbgplus-view");
  menuitem.setAttribute("hidden",false);
  menuitem.setAttribute("disabled",false);
  if (!show_view)
  { menuitem.setAttribute("hidden",true); }
  else
  { menuitem.setAttribute(attr,!showbg); }

  menuitem = document.getElementById("viewbgplus-save");
  menuitem.setAttribute("hidden",false);
  menuitem.setAttribute("disabled",false);
  if (!show_save)
  { menuitem.setAttribute("hidden",true); }
  else
  { menuitem.setAttribute(attr,!showbg); }

  menuitem = document.getElementById("viewbgplus-quicksave");
  menuitem.setAttribute("hidden",false);
  menuitem.setAttribute("disabled",false);
  if (!show_qsave || !show_save)
  { menuitem.setAttribute("hidden",true); }
  else
  { menuitem.setAttribute(attr,!showbg); }
  
  menuitem = document.getElementById("viewbgplus-quicksaveimg");
  menuitem.setAttribute("hidden",false);
  menuitem.setAttribute("disabled",false);
  if (!show_qsave)
  { menuitem.setAttribute("hidden",true); }
  else
  { menuitem.setAttribute(attr,!showimg); }
  
  // Remove built-in context menu item
  try
  { removeBuiltIn = prefs.getBoolPref("viewbgplus.remove_builtin"); }
  catch (ex) {}
  if (removeBuiltIn)
  {
   var menu = document.getElementById("contentAreaContextMenu");
   menuitem = document.getElementById("context-viewbgimage");
   menu.removeChild(menuitem);
  }
 }
} 


function init() 
{
 var menu = document.getElementById("contentAreaContextMenu");
 menu.addEventListener("popupshowing", viewbgplusContext, false);
 string_bundle = document.getElementById("viewbgplus.stringbundle");
}


window.addEventListener("load", init, false);